@@ -1,4 +1,4 @@
-## Copyright (C) 2009, 2010, José Luis García Pallero, <jgpallero@gmail.com>
+## Copyright (C) 2009-2020, José Luis García Pallero, <jgpallero@gmail.com>
 ##
 ## This file is part of OctPROJ.
 ##
@@ -20,22 +20,28 @@
 ## @deftypefn {}{[@var{lon},@var{lat}] =}op_inv(@var{X},@var{Y},@var{params})
 ##
 ## This function unprojects cartesian projected coordinates (in a defined
-## cartographic projection) into geodetic coordinates using the PROJ.4 function
-## pj_inv().
+## cartographic projection) into geodetic coordinates using the PROJ function
+## proj_trans_generic().
 ##
 ## @var{X} contains the X projected coordinates.
 ## @var{Y} contains the Y projected coordinates.
-## @var{params} is a text string containing the projection parameters in PROJ.4
-## format.
+## @var{params} is a text string containing the projection parameters in PROJ
+## format (ONLY format '+' style, see https://proj.org/usage/index.html).
 ##
-## @var{X} or @var{Y} can be scalars, vectors or matrices with equal dimensions.
+## @var{X} or @var{Y} can be scalars, vectors or 2D matrices.
+## Linear units are by default meters, although other can be specified in
+## @var{params}, so @var{X} and @var{Y} must be congruent with @var{params}.
 ##
-## @var{lon} is the geodetic longitude, in radians.
-## @var{lat} is the geodetic latitude, in radians.
+## @var{lon} is the geodetic longitude.
+## @var{lat} is the geodetic latitude.
 ##
 ## If a projection error occurs, the resultant coordinates for the affected
 ## points have both Inf value and a warning message is emitted (one for each
 ## erroneous point).
+## Angular units are by default radians, although other units can be specified
+## in @var{params}, so @var{lon} and @var{lat} will be congruent with
+## @var{params}
+##
 ## @seealso{op_fwd, op_transform}
 ## @end deftypefn
 
@@ -75,8 +81,14 @@ end
 %*******************************************************************************
 
 try
+    %check for NaN values
+    xNaN = isnan(X);
+    yNaN = isnan(Y);
     %calling oct function
     [lon,lat] = _op_inv(X,Y,params);
+    %set the originan NaN values
+    lon(xNaN) = NaN;
+    lat(yNaN) = NaN;
     %convert output vectors in matrices of adequate size
     lon = reshape(lon,rowWork,colWork);
     lat = reshape(lat,rowWork,colWork);
@@ -98,34 +110,24 @@ end
 function [a,b,rowWork,colWork] = checkInputArguments(a,b,params)
 
 %a must be matrix type
-if isnumeric(a)
-    %a dimensions
-    [rowA,colA] = size(a);
-else
+if (~isnumeric(a))||isempty(a)
     error('The first input argument is not numeric');
 end
 %b must be matrix type
-if isnumeric(b)
-    %b dimensions
-    [rowB,colB] = size(b);
-else
+if (~isnumeric(b))||isempty(b)
     error('The second input argument is not numeric');
 end
-%checking a and b dimensions
-if (rowA~=rowB)||(colA~=colB)
-    error('The dimensions of input arguments are not the same');
-else
-    %working dimensions
-    rowWork = rowA;
-    colWork = colA;
-    %convert a and b in column vectors
-    a = reshape(a,rowWork*colWork,1);
-    b = reshape(b,rowWork*colWork,1);
-end
 %params must be a text string
 if ~ischar(params)
     error('The third input argument is not a text string');
 end
+%equalize dimensions
+[a,b] = op_aux_equalize_dimensions(0,a,b);
+%dimensions
+[rowWork,colWork] = size(a);
+%convert a, b and c in column vectors
+a = reshape(a,rowWork*colWork,1);
+b = reshape(b,rowWork*colWork,1);
 
 
 
@@ -141,7 +143,7 @@ end
 
 
 %!test
-%!  [lon,lat]=op_inv(255466.98,4765182.93,'+proj=utm +lon_0=3w +ellps=GRS80');
+%!  [lon,lat]=op_inv(255466.98,4765182.93,'+proj=utm +zone=30 +lon_0=3w +ellps=GRS80');
 %!  assert(lon*180/pi,-6,1e-7)
 %!  assert(lat*180/pi,43,1e-7)
 %!error(op_inv)
@@ -149,8 +151,7 @@ end
 %!error(op_inv('string',2,3))
 %!error(op_inv(1,'string',3))
 %!error(op_inv(1,2,3))
-%!error(op_inv([1 1;2 2],2,'+proj=utm +lon_0=3w +ellps=GRS80'))
-%!error(op_inv(1,[2 2;3 3],'+proj=utm +lon_0=3w +ellps=GRS80'))
+%!error(op_inv([1 2 3],[2 2;3 3],'+proj=utm +zone=30 +lon_0=3w +ellps=GRS80'))
 
 
 
